@@ -59,7 +59,7 @@ struct vf
     uint32_t design_size; /* A fixword-pts quantity */
     unsigned int num_dev_fonts, max_dev_fonts;
     struct font_def *dev_fonts;
-    unsigned char **ch_pkt;
+    unsigned char **ch_pkt, message_flag;
     uint32_t *pkt_len;
     unsigned int num_chars;
 };
@@ -103,6 +103,7 @@ static void resize_vf_fonts(int size)
             vf_fonts[i].num_dev_fonts = 0;
             vf_fonts[i].max_dev_fonts = 0;
             vf_fonts[i].dev_fonts = NULL;
+            vf_fonts[i].message_flag = 0;
         }
         max_vf_fonts = size;
     }
@@ -414,6 +415,22 @@ void vf_set_char(int32_t ch, int vf_font)
         dvi_vf_init (default_font);
         if (ch >= vf_fonts[vf_font].num_chars ||
             !(start = (vf_fonts[vf_font].ch_pkt)[ch])) {
+            if (tfm_is_jfm(vf_fonts[vf_font].dev_fonts[0].tfm_id) &&
+                ch < 0x1000000 && dpx_conf.compat_mode != dpx_mode_xdv_mode) {
+                /* fallback multibyte character for (u)pTeX */
+                if (dpx_conf.verbose_level == 1)
+                    if (vf_fonts[vf_font].message_flag == 0) {
+                        dpx_warning("Fallback multibyte character in virtual font: VF:%s to TFM:%s",
+                        vf_fonts[vf_font].tex_name, vf_fonts[vf_font].dev_fonts[0].name);
+                        vf_fonts[vf_font].message_flag = 1;
+                    }
+                if (dpx_conf.verbose_level > 1)
+                    dpx_warning("Fallback multibyte character in virtual font: VF:%s char=0x%06x(%d) to TFM:%s",
+                vf_fonts[vf_font].tex_name, ch, ch, vf_fonts[vf_font].dev_fonts[0].name);
+                dvi_set (ch);
+                dvi_vf_finish();
+                return;
+            }
             fprintf (stderr, "\nchar=0x%x(%d)\n", ch, ch);
             fprintf (stderr, "Tried to set a nonexistent character in a virtual font");
             start = end = NULL;
